@@ -161,8 +161,8 @@ static const KeyBinding bindings_operators[] = {
 	{ "c",                  ACTION(OPERATOR_CHANGE)                     },
 	{ "d",                  ACTION(OPERATOR_DELETE)                     },
 	{ "g~",                 ALIAS(":|tr '[:lower:][:upper:]' '[:upper:][:lower:]'<Enter>") },
-	{ "gu",                 ALIAS(":|tr '[:upper:]' '[:lower:]'<Enter>")},
-	{ "gU",                 ALIAS(":|tr '[:lower:]' '[:upper:]'<Enter>")},
+	{ "gu",                 ALIAS(":|awk '{printf \"%s\", tolower($0)}'<Enter>")},
+	{ "gU",                 ALIAS(":|awk '{printf \"%s\", toupper($0)}'<Enter>")},
 	{ "p",                  ACTION(PUT_AFTER)                           },
 	{ "P",                  ACTION(PUT_BEFORE)                          },
 	{ "y",                  ACTION(OPERATOR_YANK)                       },
