@@ -2229,6 +2229,7 @@ int main(int argc, char *argv[]) {
 		.win_highlight = vis_lua_win_highlight,
 		.win_status = vis_lua_win_status,
 		.term_csi = vis_lua_term_csi,
+		.ui_draw = vis_lua_ui_draw,
 	};
 
 	vis = vis_new(ui_term_new(), &event);
@@ -2332,7 +2333,7 @@ int main(int argc, char *argv[]) {
 			cmd = argv[i] + (argv[i][1] == '/' || argv[i][1] == '?');
 			continue;
 		} else if (!vis_window_new(vis, argv[i])) {
-			vis_die(vis, "Can not load `%s': %s\n", argv[i], strerror(errno));
+			vis_die(vis, "Can not load '%s': %s\n", argv[i], strerror(errno));
 		}
 		win_created = true;
 		if (cmd) {
