@@ -292,6 +292,7 @@ enum {
 	OPTION_SHOW_TABS,
 	OPTION_SHOW_NEWLINES,
 	OPTION_SHOW_EOF,
+	OPTION_STATUSBAR,
 	OPTION_NUMBER,
 	OPTION_NUMBER_RELATIVE,
 	OPTION_CURSOR_LINE,
@@ -301,6 +302,8 @@ enum {
 	OPTION_CHANGE_256COLORS,
 	OPTION_LAYOUT,
 	OPTION_IGNORECASE,
+	OPTION_BREAKAT,
+	OPTION_WRAP_COLUMN,
 };
 
 static const OptionDef options[] = {
@@ -321,33 +324,46 @@ static const OptionDef options[] = {
 	},
 	[OPTION_EXPANDTAB] = {
 		{ "expandtab", "et" },
-		VIS_OPTION_TYPE_BOOL,
+		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW,
 		VIS_HELP("Replace entered <Tab> with `tabwidth` spaces")
 	},
 	[OPTION_TABWIDTH] = {
 		{ "tabwidth", "tw" },
-		VIS_OPTION_TYPE_NUMBER,
+		VIS_OPTION_TYPE_NUMBER|VIS_OPTION_NEED_WINDOW,
 		VIS_HELP("Number of spaces to display (and insert if `expandtab` is enabled) for a tab")
 	},
 	[OPTION_SHOW_SPACES] = {
-		{ "show-spaces" },
-		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW,
+		{ "showspaces", "show-spaces" },
+		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW|VIS_OPTION_DEPRECATED,
 		VIS_HELP("Display replacement symbol instead of a space")
+		NULL,
+		"show-spaces"
 	},
 	[OPTION_SHOW_TABS] = {
-		{ "show-tabs" },
-		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW,
+		{ "showtabs", "show-tabs" },
+		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW|VIS_OPTION_DEPRECATED,
 		VIS_HELP("Display replacement symbol for tabs")
+		NULL,
+		"show-tabs"
 	},
 	[OPTION_SHOW_NEWLINES] = {
-		{ "show-newlines" },
-		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW,
+		{ "shownewlines", "show-newlines" },
+		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW|VIS_OPTION_DEPRECATED,
 		VIS_HELP("Display replacement symbol for newlines")
+		NULL,
+		"show-newlines"
 	},
 	[OPTION_SHOW_EOF] = {
-		{ "show-eof" },
-		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW,
+		{ "showeof", "show-eof" },
+		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW|VIS_OPTION_DEPRECATED,
 		VIS_HELP("Display replacement symbol for lines after the end of the file")
+		NULL,
+		"show-eof"
+	},
+	[OPTION_STATUSBAR] = {
+		{ "statusbar", "sb" },
+		VIS_OPTION_TYPE_BOOL|VIS_OPTION_NEED_WINDOW,
+		VIS_HELP("Display status bar")
 	},
 	[OPTION_NUMBER] = {
 		{ "numbers", "nu" },
@@ -380,9 +396,11 @@ static const OptionDef options[] = {
 		VIS_HELP("How to load existing files 'auto', 'read' or 'mmap'")
 	},
 	[OPTION_CHANGE_256COLORS] = {
-		{ "change-256colors" },
-		VIS_OPTION_TYPE_BOOL,
+		{ "change256colors", "change-256colors" },
+		VIS_OPTION_TYPE_BOOL|VIS_OPTION_DEPRECATED,
 		VIS_HELP("Change 256 color palette to support 24bit colors")
+		NULL,
+		"change-256colors"
 	},
 	[OPTION_LAYOUT] = {
 		{ "layout" },
@@ -394,6 +412,16 @@ static const OptionDef options[] = {
 		VIS_OPTION_TYPE_BOOL,
 		VIS_HELP("Ignore case when searching")
 	},
+	[OPTION_BREAKAT] = {
+		{ "breakat", "brk" },
+		VIS_OPTION_TYPE_STRING|VIS_OPTION_NEED_WINDOW,
+		VIS_HELP("Characters which might cause a word wrap")
+	},
+	[OPTION_WRAP_COLUMN] = {
+		{ "wrapcolumn", "wc" },
+		VIS_OPTION_TYPE_NUMBER|VIS_OPTION_NEED_WINDOW,
+		VIS_HELP("Wrap lines at minimum of window width and wrapcolumn")
+	},
 };
 
 bool sam_init(Vis *vis) {
@@ -655,7 +683,7 @@ static void parse_argv(const char **s, const char *argv[], size_t maxarg) {
 
 static bool valid_cmdname(const char *s) {
 	unsigned char c = (unsigned char)*s;
-	return c && !isspace(c) && !isdigit(c) && (!ispunct(c) || (c == '-' && valid_cmdname(s+1)));
+	return c && !isspace(c) && !isdigit(c) && (!ispunct(c) || c == '_' || (c == '-' && valid_cmdname(s+1)));
 }
 
 static char *parse_cmdname(const char **s) {
@@ -1475,7 +1503,7 @@ static int extract(Vis *vis, Win *win, Command *cmd, const char *argv[], Selecti
 }
 
 static bool cmd_extract(Vis *vis, Win *win, Command *cmd, const char *argv[], Selection *sel, Filerange *range) {
-	if (!win)
+	if (!win || !text_range_valid(range))
 		return false;
 	int matches = 0;
 	if (count_negative(cmd->cmd))
@@ -1564,14 +1592,16 @@ static bool cmd_print(Vis *vis, Win *win, Command *cmd, const char *argv[], Sele
 
 static bool cmd_files(Vis *vis, Win *win, Command *cmd, const char *argv[], Selection *sel, Filerange *range) {
 	bool ret = true;
-	for (Win *win = vis->windows; win; win = win->next) {
-		if (win->file->internal)
+	for (Win *wn, *w = vis->windows; w; w = wn) {
+		/* w can get freed by sam_execute() so store w->next early */
+		wn = w->next;
+		if (w->file->internal)
 			continue;
 		bool match = !cmd->regex ||
-		             (win->file->name && text_regex_match(cmd->regex, win->file->name, 0) == 0);
+		             (w->file->name && text_regex_match(cmd->regex, w->file->name, 0) == 0);
 		if (match ^ (argv[0][0] == 'Y')) {
 			Filerange def = text_range_new(0, 0);
-			ret &= sam_execute(vis, win, cmd->cmd, NULL, &def);
+			ret &= sam_execute(vis, w, cmd->cmd, NULL, &def);
 		}
 	}
 	return ret;
@@ -1737,7 +1767,7 @@ static bool cmd_filter(Vis *vis, Win *win, Command *cmd, const char *argv[], Sel
 	buffer_init(&bufout);
 	buffer_init(&buferr);
 
-	int status = vis_pipe(vis, win->file, range, &argv[1], &bufout, read_buffer, &buferr, read_buffer);
+	int status = vis_pipe(vis, win->file, range, &argv[1], &bufout, read_buffer, &buferr, read_buffer, false);
 
 	if (vis->interrupted) {
 		vis_info_show(vis, "Command cancelled");
@@ -1777,7 +1807,7 @@ static bool cmd_pipeout(Vis *vis, Win *win, Command *cmd, const char *argv[], Se
 	Buffer buferr;
 	buffer_init(&buferr);
 
-	int status = vis_pipe(vis, win->file, range, (const char*[]){ argv[1], NULL }, NULL, NULL, &buferr, read_buffer);
+	int status = vis_pipe(vis, win->file, range, (const char*[]){ argv[1], NULL }, NULL, NULL, &buferr, read_buffer, false);
 
 	if (vis->interrupted)
 		vis_info_show(vis, "Command cancelled");
