@@ -129,12 +129,14 @@ static void windows_arrange(Vis *vis, enum UiLayout layout) {
 	vis->ui->arrange(vis->ui, layout);
 }
 
-static void tabwidth_set(Vis *vis, int tabwidth) {
-	if (tabwidth < 1 || tabwidth > 8)
-		return;
-	for (Win *win = vis->windows; win; win = win->next)
-		view_tabwidth_set(win->view, tabwidth);
-	vis->tabwidth = tabwidth;
+void vis_shell_set(Vis *vis, const char *new_shell) {
+	char *shell =  strdup(new_shell);
+	if (!shell) {
+		vis_info_show(vis, "Failed to change shell");
+	} else {
+		free(vis->shell);
+		vis->shell = shell;
+	}
 }
 
 /* parse human-readable boolean value in s. If successful, store the result in
@@ -175,6 +177,9 @@ static bool cmd_set(Vis *vis, Win *win, Command *cmd, const char *argv[], Select
 		return false;
 	}
 
+	if (opt->flags & VIS_OPTION_DEPRECATED && strcmp(opt->context, name) == 0)
+		vis_info_show(vis, "%s is deprecated and will be removed in the next release", name);
+
 	if (!win && (opt->flags & VIS_OPTION_NEED_WINDOW)) {
 		vis_info_show(vis, "Need active window for `:set %s'", name);
 		return false;
@@ -241,16 +246,8 @@ static bool cmd_set(Vis *vis, Win *win, Command *cmd, const char *argv[], Select
 
 	switch (opt_index) {
 	case OPTION_SHELL:
-	{
-		char *shell = strdup(arg.s);
-		if (!shell) {
-			vis_info_show(vis, "Failed to change shell");
-			return false;
-		}
-		free(vis->shell);
-		vis->shell = shell;
+		vis_shell_set(vis, arg.s);
 		break;
-	}
 	case OPTION_ESCDELAY:
 	{
 		TermKey *termkey = vis->ui->termkey_get(vis->ui);
@@ -258,24 +255,26 @@ static bool cmd_set(Vis *vis, Win *win, Command *cmd, const char *argv[], Select
 		break;
 	}
 	case OPTION_EXPANDTAB:
-		vis->expandtab = toggle ? !vis->expandtab : arg.b;
+		vis->win->expandtab = toggle ? !vis->win->expandtab : arg.b;
 		break;
 	case OPTION_AUTOINDENT:
 		vis->autoindent = toggle ? !vis->autoindent : arg.b;
 		break;
 	case OPTION_TABWIDTH:
-		tabwidth_set(vis, arg.i);
+		view_tabwidth_set(vis->win->view, arg.i);
 		break;
 	case OPTION_SHOW_SPACES:
 	case OPTION_SHOW_TABS:
 	case OPTION_SHOW_NEWLINES:
 	case OPTION_SHOW_EOF:
+	case OPTION_STATUSBAR:
 	{
 		const int values[] = {
 			[OPTION_SHOW_SPACES] = UI_OPTION_SYMBOL_SPACE,
 			[OPTION_SHOW_TABS] = UI_OPTION_SYMBOL_TAB|UI_OPTION_SYMBOL_TAB_FILL,
 			[OPTION_SHOW_NEWLINES] = UI_OPTION_SYMBOL_EOL,
 			[OPTION_SHOW_EOF] = UI_OPTION_SYMBOL_EOF,
+			[OPTION_STATUSBAR] = UI_OPTION_STATUSBAR,
 		};
 		int flags = view_options_get(win->view);
 		if (arg.b || (toggle && !(flags & values[opt_index])))
@@ -364,6 +363,15 @@ static bool cmd_set(Vis *vis, Win *win, Command *cmd, const char *argv[], Select
 	case OPTION_IGNORECASE:
 		vis->ignorecase = toggle ? !vis->ignorecase : arg.b;
 		break;
+	case OPTION_BREAKAT:
+		if (!view_breakat_set(win->view, arg.s)) {
+			vis_info_show(vis, "Failed to set breakat");
+			return false;
+		}
+		break;
+	case OPTION_WRAP_COLUMN:
+		view_wrapcolumn_set(win->view, arg.i);
+		break;
 	default:
 		if (!opt->func)
 			return false;
@@ -379,7 +387,11 @@ static bool is_file_pattern(const char *pattern) {
 	struct stat meta;
 	if (stat(pattern, &meta) == 0 && S_ISDIR(meta.st_mode))
 		return true;
-	for (char special[] = "*?[{$~", *s = special; *s; s++) {
+	/* tilde expansion is defined only for the tilde at the
+	   beginning of the pattern. */
+	if (pattern[0] == '~')
+		return true;
+	for (char special[] = "*?[{$", *s = special; *s; s++) {
 		if (strchr(pattern, *s))
 			return true;
 	}
@@ -404,7 +416,7 @@ static const char *file_open_dialog(Vis *vis, const char *pattern) {
 	Filerange empty = text_range_new(0,0);
 	int status = vis_pipe(vis, vis->win->file, &empty,
 		(const char*[]){ buffer_content0(&bufcmd), NULL },
-		&bufout, read_buffer, &buferr, read_buffer);
+		&bufout, read_buffer, &buferr, read_buffer, false);
 
 	if (status == 0)
 		strncpy(name, buffer_content0(&bufout), sizeof(name)-1);
@@ -628,8 +640,27 @@ static bool cmd_earlier_later(Vis *vis, Win *win, Command *cmd, const char *argv
 	return pos != EPOS;
 }
 
+static int space_replace(char *dest, const char *src, size_t dlen) {
+	int invisiblebytes = 0;
+	size_t i, size = LENGTH("␣") - 1;
+	for (i = 0; *src && i < dlen; src++) {
+		if (*src == ' ' && i < dlen - size - 1) {
+			memcpy(&dest[i], "␣", size);
+			i += size;
+			invisiblebytes += size - 1;
+		} else {
+			dest[i] = *src;
+			i++;
+		}
+	}
+	dest[i] = '\0';
+	return invisiblebytes;
+}
+
 static bool print_keylayout(const char *key, void *value, void *data) {
-	return text_appendf(data, "  %-18s\t%s\n", key[0] == ' ' ? "␣" : key, (char*)value);
+	char buf[64];
+	int invisiblebytes = space_replace(buf, key, sizeof(buf));
+	return text_appendf(data, "  %-*s\t%s\n", 18+invisiblebytes, buf, (char*)value);
 }
 
 static bool print_keybinding(const char *key, void *value, void *data) {
@@ -637,7 +668,9 @@ static bool print_keybinding(const char *key, void *value, void *data) {
 	const char *desc = binding->alias;
 	if (!desc && binding->action)
 		desc = VIS_HELP_USE(binding->action->help);
-	return text_appendf(data, "  %-18s\t%s\n", key[0] == ' ' ? "␣" : key, desc ? desc : "");
+	char buf[64];
+	int invisiblebytes = space_replace(buf, key, sizeof(buf));
+	return text_appendf(data, "  %-*s\t%s\n", 18+invisiblebytes, buf, desc ? desc : "");
 }
 
 static void print_mode(Mode *mode, Text *txt) {
