@@ -5,11 +5,14 @@
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
+
 #else
 typedef struct lua_State lua_State;
+typedef void* lua_CFunction;
 #endif
 
 #include "vis.h"
+#include "vis-subprocess.h"
 
 /* add a directory to consider when loading lua files */
 bool vis_lua_path_add(Vis*, const char *path);
@@ -38,5 +41,7 @@ void vis_lua_win_close(Vis*, Win*);
 void vis_lua_win_highlight(Vis*, Win*);
 void vis_lua_win_status(Vis*, Win*);
 void vis_lua_term_csi(Vis*, const long *);
+void vis_lua_process_response(Vis *, const char *, char *, size_t, ResponseType);
+void vis_lua_ui_draw(Vis*);
 
 #endif
