@@ -28,6 +28,7 @@
 #include "vis-core.h"
 #include "sam.h"
 #include "ui.h"
+#include "vis-subprocess.h"
 
 
 static void macro_replay(Vis *vis, const Macro *macro);
@@ -104,6 +105,10 @@ bool vis_event_emit(Vis *vis, enum VisEvents id, ...) {
 		if (vis->event->term_csi)
 			vis->event->term_csi(vis, va_arg(ap, const long *));
 		break;
+	case VIS_EVENT_UI_DRAW:
+		if (vis->event->ui_draw)
+			vis->event->ui_draw(vis);
+		break;
 	}
 
 	va_end(ap);
@@ -179,16 +184,30 @@ err:
 
 static File *file_new(Vis *vis, const char *name) {
 	char *name_absolute = NULL;
+	bool cmp_names = 0;
+	struct stat new;
+
 	if (name) {
 		if (!(name_absolute = absolute_path(name)))
 			return NULL;
+
+		if (stat(name_absolute, &new)) {
+			if (errno != ENOENT) {
+				free(name_absolute);
+				return NULL;
+			}
+			cmp_names = 1;
+		}
+
 		File *existing = NULL;
-		/* try to detect whether the same file is already open in another window
-		 * TODO: do this based on inodes */
+		/* try to detect whether the same file is already open in another window */
 		for (File *file = vis->files; file; file = file->next) {
-			if (file->name && strcmp(file->name, name_absolute) == 0) {
-				existing = file;
-				break;
+			if (file->name) {
+				if ((cmp_names && strcmp(file->name, name_absolute) == 0) ||
+				    (file->stat.st_dev == new.st_dev && file->stat.st_ino == new.st_ino)) {
+					existing = file;
+					break;
+				}
 			}
 		}
 		if (existing) {
@@ -282,7 +301,6 @@ static void window_draw_colorcolumn(Win *win) {
 	int cc = view_colorcolumn_get(view);
 	if (cc <= 0)
 		return;
-	CellStyle style = win->ui->style_get(win->ui, UI_STYLE_COLOR_COLUMN);
 	size_t lineno = 0;
 	int line_cols = 0; /* Track the number of columns we've passed on each line */
 	bool line_cc_set = false; /* Has the colorcolumn attribute been set for this line yet */
@@ -300,10 +318,7 @@ static void window_draw_colorcolumn(Win *win) {
 
 		/* This screen line contains the cell we want to highlight */
 		if (cc <= line_cols + width) {
-			CellStyle *orig = &l->cells[cc - 1 - line_cols].style;
-			orig->attr = style.attr;
-			orig->fg = is_default_color(style.fg) ? orig->fg : style.fg;
-			orig->bg = is_default_color(style.bg) ? orig->bg : style.bg;
+			win->ui->style_set(win->ui, &l->cells[cc - 1 - line_cols], UI_STYLE_COLOR_COLUMN);
 			line_cc_set = true;
 		} else {
 			line_cols += width;
@@ -323,22 +338,20 @@ static void window_draw_cursorline(Win *win) {
 		return;
 
 	int width = view_width_get(view);
-	CellStyle style = win->ui->style_get(win->ui, UI_STYLE_CURSOR_LINE);
 	Selection *sel = view_selections_primary_get(view);
 	size_t lineno = view_cursors_line_get(sel)->lineno;
 	for (Line *l = view_lines_first(view); l; l = l->next) {
 		if (l->lineno == lineno) {
-			for (int x = 0; x < width; x++) {
-				l->cells[x].style.attr |= style.attr;
-				l->cells[x].style.bg = style.bg;
-			}
+			for (int x = 0; x < width; x++)
+				win->ui->style_set(win->ui, &l->cells[x], UI_STYLE_CURSOR_LINE);
 		} else if (l->lineno > lineno) {
 			break;
 		}
 	}
 }
 
-static void window_draw_selection(View *view, Selection *cur, CellStyle *style) {
+static void window_draw_selection(Win *win, Selection *cur) {
+	View *view = win->view;
 	Filerange sel = view_selections_get(cur);
 	if (!text_range_valid(&sel))
 		return;
@@ -359,24 +372,12 @@ static void window_draw_selection(View *view, Selection *cur, CellStyle *style)
 	for (Line *l = start_line; l != end_line->next; l = l->next) {
 		int col = (l == start_line) ? start_col : 0;
 		int end = (l == end_line) ? end_col : l->width;
-		while (col < end) {
-			if (cell_color_equal(l->cells[col].style.fg, style->bg)) {
-				CellStyle old = l->cells[col].style;
-				if (!cell_color_equal(old.fg, old.bg)) {
-					l->cells[col].style.fg = old.bg;
-					l->cells[col].style.bg = old.fg;
-				} else {
-					l->cells[col].style.attr = style->attr;
-				}
-			} else {
-				l->cells[col].style.bg = style->bg;
-			}
-			col++;
-		}
+		while (col < end)
+			win->ui->style_set(win->ui, &l->cells[col++], UI_STYLE_SELECTION);
 	}
 }
 
-static void window_draw_cursor_matching(Win *win, Selection *cur, CellStyle *style) {
+static void window_draw_cursor_matching(Win *win, Selection *cur) {
 	if (win->vis->mode->visual)
 		return;
 	Line *line_match; int col_match;
@@ -387,24 +388,18 @@ static void window_draw_cursor_matching(Win *win, Selection *cur, CellStyle *sty
 		return;
 	if (!view_coord_get(win->view, pos_match, &line_match, NULL, &col_match))
 		return;
-	if (cell_color_equal(line_match->cells[col_match].style.fg, style->fg)) {
-		CellStyle old = line_match->cells[col_match].style;
-		line_match->cells[col_match].style.fg = old.bg;
-		line_match->cells[col_match].style.bg = old.fg;
-	} else {
-		line_match->cells[col_match].style.bg = style->bg;
-	}
+	win->ui->style_set(win->ui, &line_match->cells[col_match], UI_STYLE_SELECTION);
 }
 
-static void window_draw_cursor(Win *win, Selection *cur, CellStyle *style, CellStyle *sel_style) {
+static void window_draw_cursor(Win *win, Selection *cur) {
 	if (win->vis->win != win)
 		return;
 	Line *line = view_cursors_line_get(cur);
 	int col = view_cursors_cell_get(cur);
 	if (!line || col == -1)
 		return;
-	line->cells[col].style = *style;
-	window_draw_cursor_matching(win, cur, sel_style);
+	win->ui->style_set(win->ui, &line->cells[col], UI_STYLE_CURSOR);
+	window_draw_cursor_matching(win, cur);
 	return;
 }
 
@@ -412,24 +407,21 @@ static void window_draw_selections(Win *win) {
 	View *view = win->view;
 	Filerange viewport = view_viewport_get(view);
 	Selection *sel = view_selections_primary_get(view);
-	CellStyle style_cursor = win->ui->style_get(win->ui, UI_STYLE_CURSOR);
-	CellStyle style_cursor_primary = win->ui->style_get(win->ui, UI_STYLE_CURSOR_PRIMARY);
-	CellStyle style_selection = win->ui->style_get(win->ui, UI_STYLE_SELECTION);
 	for (Selection *s = view_selections_prev(sel); s; s = view_selections_prev(s)) {
-		window_draw_selection(win->view, s, &style_selection);
+		window_draw_selection(win, s);
 		size_t pos = view_cursors_pos(s);
 		if (pos < viewport.start)
 			break;
-		window_draw_cursor(win, s, &style_cursor, &style_selection);
+		window_draw_cursor(win, s);
 	}
-	window_draw_selection(win->view, sel, &style_selection);
-	window_draw_cursor(win, sel, &style_cursor_primary, &style_selection);
+	window_draw_selection(win, sel);
+	window_draw_cursor(win, sel);
 	for (Selection *s = view_selections_next(sel); s; s = view_selections_next(s)) {
-		window_draw_selection(win->view, s, &style_selection);
+		window_draw_selection(win, s);
 		size_t pos = view_cursors_pos(s);
 		if (pos > viewport.end)
 			break;
-		window_draw_cursor(win, s, &style_cursor, &style_selection);
+		window_draw_cursor(win, s);
 	}
 }
 
@@ -437,10 +429,9 @@ static void window_draw_eof(Win *win) {
 	View *view = win->view;
 	if (view_width_get(view) == 0)
 		return;
-	CellStyle style = win->ui->style_get(win->ui, UI_STYLE_EOF);
 	for (Line *l = view_lines_last(view)->next; l; l = l->next) {
 		strncpy(l->cells[0].data, view_symbol_eof_get(view), sizeof(l->cells[0].data)-1);
-		l->cells[0].style = style;
+		win->ui->style_set(win->ui, &l->cells[0], UI_STYLE_EOF);
 	}
 }
 
@@ -474,6 +465,7 @@ Win *window_new_file(Vis *vis, File *file, enum UiOption options) {
 	win->vis = vis;
 	win->file = file;
 	win->view = view_new(file->text);
+	win->expandtab = false;
 	win->ui = vis->ui->window_new(vis->ui, win, options);
 	if (!win->view || !win->ui) {
 		window_free(win);
@@ -483,7 +475,6 @@ Win *window_new_file(Vis *vis, File *file, enum UiOption options) {
 	mark_init(&win->saved_selections);
 	file->refcount++;
 	view_options_set(win->view, view_options_get(win->view));
-	view_tabwidth_set(win->view, vis->tabwidth);
 
 	if (vis->windows)
 		vis->windows->prev = win;
@@ -515,6 +506,7 @@ bool vis_window_reload(Win *win) {
 }
 
 bool vis_window_split(Win *original) {
+	vis_doupdates(original->vis, false);
 	Win *win = window_new_file(original->vis, original->file, UI_OPTION_STATUSBAR);
 	if (!win)
 		return false;
@@ -527,6 +519,7 @@ bool vis_window_split(Win *original) {
 	win->file = original->file;
 	view_options_set(win->view, view_options_get(original->view));
 	view_cursor_to(win->view, view_cursor_get(original->view));
+	vis_doupdates(win->vis, true);
 	return true;
 }
 
@@ -585,15 +578,21 @@ void vis_resume(Vis *vis) {
 	vis->ui->resume(vis->ui);
 }
 
+void vis_doupdates(Vis *vis, bool doupdate) {
+	vis->ui->doupdates(vis->ui, doupdate);
+}
+
 bool vis_window_new(Vis *vis, const char *filename) {
 	File *file = file_new(vis, filename);
 	if (!file)
 		return false;
+	vis_doupdates(vis, false);
 	Win *win = window_new_file(vis, file, UI_OPTION_STATUSBAR|UI_OPTION_SYMBOL_EOF);
 	if (!win) {
 		file_free(vis, file);
 		return false;
 	}
+	vis_doupdates(vis, true);
 
 	return true;
 }
@@ -672,8 +671,6 @@ Vis *vis_new(Ui *ui, VisEvent *event) {
 		return NULL;
 	vis->exit_status = -1;
 	vis->ui = ui;
-	vis->tabwidth = 8;
-	vis->expandtab = false;
 	vis->change_colors = true;
 	for (size_t i = 0; i < LENGTH(vis->registers); i++)
 		register_init(&vis->registers[i]);
@@ -962,6 +959,9 @@ void vis_do(Vis *vis) {
 				if (a->textobj->type & TEXTOBJECT_DELIMITED_OUTER) {
 					r.start--;
 					r.end++;
+				} else if (linewise && (a->textobj->type & TEXTOBJECT_DELIMITED_INNER)) {
+					r.start = text_line_next(txt, r.start);
+					r.end = text_line_prev(txt, r.end);
 				}
 
 				if (vis->mode->visual || (i > 0 && !(a->textobj->type & TEXTOBJECT_NON_CONTIGUOUS)))
@@ -1213,7 +1213,7 @@ static void vis_keys_process(Vis *vis, size_t pos) {
 		*end = tmp;
 
 		if (prefix) {
-			/* input sofar is ambigious, wait for more */
+			/* input so far is ambiguous, wait for more */
 			cur = end;
 			end = start;
 		} else if (binding) { /* exact match */
@@ -1412,7 +1412,8 @@ int vis_run(Vis *vis) {
 
 		vis_update(vis);
 		idle.tv_sec = vis->mode->idle_timeout;
-		int r = pselect(1, &fds, NULL, NULL, timeout, &emptyset);
+		int r = pselect(vis_process_before_tick(&fds) + 1, &fds, NULL, NULL,
+		                timeout, &emptyset);
 		if (r == -1 && errno == EINTR)
 			continue;
 
@@ -1420,6 +1421,7 @@ int vis_run(Vis *vis) {
 			/* TODO save all pending changes to a ~suffixed file */
 			vis_die(vis, "Error in mainloop: %s\n", strerror(errno));
 		}
+		vis_process_tick(vis, &fds);
 
 		if (!FD_ISSET(STDIN_FILENO, &fds)) {
 			if (vis->mode->idle)
@@ -1618,12 +1620,12 @@ void vis_insert_tab(Vis *vis) {
 	Win *win = vis->win;
 	if (!win)
 		return;
-	if (!vis->expandtab) {
+	if (!win->expandtab) {
 		vis_insert_key(vis, "\t", 1);
 		return;
 	}
 	char spaces[9];
-	int tabwidth = MIN(vis->tabwidth, LENGTH(spaces) - 1);
+	int tabwidth = MIN(view_tabwidth_get(vis->win->view), LENGTH(spaces) - 1);
 	for (Selection *s = view_selections(win->view); s; s = view_selections_next(s)) {
 		size_t pos = view_cursors_pos(s);
 		int width = text_line_width_get(win->file->text, pos);
@@ -1712,7 +1714,8 @@ Regex *vis_regex(Vis *vis, const char *pattern) {
 
 int vis_pipe(Vis *vis, File *file, Filerange *range, const char *argv[],
 	void *stdout_context, ssize_t (*read_stdout)(void *stdout_context, char *data, size_t len),
-	void *stderr_context, ssize_t (*read_stderr)(void *stderr_context, char *data, size_t len)) {
+	void *stderr_context, ssize_t (*read_stderr)(void *stderr_context, char *data, size_t len),
+	bool fullscreen) {
 
 	/* if an invalid range was given, stdin (i.e. key board input) is passed
 	 * through the external command. */
@@ -1737,7 +1740,7 @@ int vis_pipe(Vis *vis, File *file, Filerange *range, const char *argv[],
 		return -1;
 	}
 
-	vis->ui->terminal_save(vis->ui);
+	vis->ui->terminal_save(vis->ui, fullscreen);
 	pid_t pid = fork();
 
 	if (pid == -1) {
@@ -1933,7 +1936,10 @@ err:
 	vis->interrupted = false;
 	vis->ui->terminal_restore(vis->ui);
 
-	return status;
+	if (WIFEXITED(status))
+		return WEXITSTATUS(status);
+
+	return -1;
 }
 
 static ssize_t read_buffer(void *context, char *data, size_t len) {
@@ -1941,13 +1947,14 @@ static ssize_t read_buffer(void *context, char *data, size_t len) {
 	return len;
 }
 
-int vis_pipe_collect(Vis *vis, File *file, Filerange *range, const char *argv[], char **out, char **err) {
+int vis_pipe_collect(Vis *vis, File *file, Filerange *range, const char *argv[], char **out, char **err, bool fullscreen) {
 	Buffer bufout, buferr;
 	buffer_init(&bufout);
 	buffer_init(&buferr);
 	int status = vis_pipe(vis, file, range, argv,
 	                      &bufout, out ? read_buffer : NULL,
-	                      &buferr, err ? read_buffer : NULL);
+	                      &buferr, err ? read_buffer : NULL,
+	                      fullscreen);
 	buffer_terminate(&bufout);
 	buffer_terminate(&buferr);
 	if (out)
