@@ -58,6 +58,7 @@ typedef struct {
 	void (*win_highlight)(Vis*, Win*);
 	void (*win_status)(Vis*, Win*);
 	void (*term_csi)(Vis*, const long *);
+	void (*ui_draw)(Vis*);
 } VisEvent;
 
 /** Union used to pass arguments to key action functions. */
@@ -68,7 +69,6 @@ typedef union {
 	const void *v;
 	void (*w)(View*);
 	void (*f)(Vis*);
-	Filerange (*combine)(const Filerange*, const Filerange*);
 } Arg;
 
 /**
@@ -116,7 +116,7 @@ Vis *vis_new(Ui*, VisEvent*);
 /** Free all resources associated with this editor instance, terminates UI. */
 void vis_free(Vis*);
 /**
- * Enter main loop, start processing user input. 
+ * Enter main loop, start processing user input.
  * @return The editor exit status code.
  */
 int vis_run(Vis*);
@@ -145,7 +145,14 @@ void vis_suspend(Vis*);
  */
 void vis_resume(Vis*);
 /**
- * Inform the editor core that a signal occured.
+ * Set doupdate flag.
+ * @rst
+ * .. note:: Prevent flickering in curses by delaying window updates.
+ * @endrst
+ */
+void vis_doupdates(Vis*, bool);
+/**
+ * Inform the editor core that a signal occurred.
  * @return Whether the signal was handled.
  * @rst
  * .. note:: Being designed as a library the editor core does *not* register any
@@ -185,12 +192,10 @@ void vis_update(Vis*);
  */
 /**
  * Create a new window and load the given file.
- * @param filename If ``NULL`` a unamed, empty buffer is created.
+ * @param filename If ``NULL`` an unnamed, empty buffer is created.
  * @rst
  * .. note:: If the given file name is already opened in another window,
  *           the underlying File object is shared.
- * .. warning:: This duplication detection is currently based on normalized,
- *              absolute file names. TODO: compare inodes instead.
  * @endrst
  */
 bool vis_window_new(Vis*, const char *filename);
@@ -251,8 +256,6 @@ void vis_info_hide(Vis*);
 
 /** Display arbitrary long message in a dedicated window. */
 void vis_message_show(Vis*, const char *msg);
-/** Close message window. */
-void vis_message_hide(Vis*);
 /**
  * @}
  * @defgroup vis_changes
@@ -425,7 +428,7 @@ int vis_operator_register(Vis*, VisOperatorFunction*, void *context);
  *  - A visual mode is active.
  *  - The same operator was already set (range will be the current line).
  *
- * Otherwise the operator will be executed on the range determinded by:
+ * Otherwise the operator will be executed on the range determined by:
  *  - A motion (see `vis_motion`).
  *  - A text object (`vis_textobject`).
  *
@@ -582,6 +585,10 @@ int vis_count_get(Vis*);
 int vis_count_get_default(Vis*, int def);
 /** Set a count. */
 void vis_count_set(Vis*, int count);
+/** Set the tabwidth */
+void vis_tabwidth_set(Vis*, int tw);
+/** Set the shell */
+void vis_shell_set(Vis*, const char *new_shell);
 
 typedef struct {
 	Vis *vis;
@@ -597,7 +604,7 @@ VisCountIterator vis_count_iterator_init(Vis*, int count);
  * Increment iterator counter.
  * @return Whether iteration should continue.
  * @rst
- * .. note:: Terminates iteration if the edtior was
+ * .. note:: Terminates iteration if the editor was
  *           `interrupted <vis_interrupt>`_ in the meantime.
  * @endrst
  */
@@ -851,6 +858,7 @@ enum VisOption {
 	VIS_OPTION_TYPE_NUMBER = 1 << 2,
 	VIS_OPTION_VALUE_OPTIONAL = 1 << 3,
 	VIS_OPTION_NEED_WINDOW = 1 << 4,
+	VIS_OPTION_DEPRECATED = 1 << 5,
 };
 
 /**
@@ -903,6 +911,10 @@ bool vis_prompt_cmd(Vis*, const char *cmd);
  * If the ``read_stdout`` and ``read_stderr`` callbacks are non-NULL they
  * will be invoked when output from the forked process is available.
  *
+ * If ``fullscreen`` is set to ``true`` the external process is assumed to
+ * be a fullscreen program (e.g. curses based) and the ui context is
+ * restored accordingly.
+ *
  * @rst
  * .. warning:: The editor core is blocked until this function returns.
  * @endrst
@@ -911,7 +923,8 @@ bool vis_prompt_cmd(Vis*, const char *cmd);
  */
 int vis_pipe(Vis*, File*, Filerange*, const char *argv[],
 	void *stdout_context, ssize_t (*read_stdout)(void *stdout_context, char *data, size_t len),
-	void *stderr_context, ssize_t (*read_stderr)(void *stderr_context, char *data, size_t len));
+	void *stderr_context, ssize_t (*read_stderr)(void *stderr_context, char *data, size_t len),
+	bool fullscreen);
 
 /**
  * Pipe a Filerange to an external process, return its exit status and capture
@@ -919,12 +932,13 @@ int vis_pipe(Vis*, File*, Filerange*, const char *argv[],
  * @param argv Argument list, must be ``NULL`` terminated.
  * @param out Data written to ``stdout``, will be ``NUL`` terminated.
  * @param err Data written to ``stderr``, will be ``NUL`` terminated.
+ * @param fullscreen Whether the external process is a fullscreen program (e.g. curses based)
  * @rst
  * .. warning:: The pointers stored in ``out`` and ``err`` need to be `free(3)`-ed
  *              by the caller.
  * @endrst
  */
-int vis_pipe_collect(Vis*, File*, Filerange*, const char *argv[], char **out, char **err);
+int vis_pipe_collect(Vis*, File*, Filerange*, const char *argv[], char **out, char **err, bool fullscreen);
 
 /**
  * @}
